@@ -1,36 +1,42 @@
 # mypy: disable-error-code=type-abstract
 import logging
 import warnings
-from functools import partial
+from collections import Counter
 from typing import Optional, TypeVar, List, TYPE_CHECKING
 
 from feeluown.media import Media
 from feeluown.utils.aio import run_fn, as_completed
 from feeluown.utils.dispatch import Signal
-from .base import SearchType, ModelType
-from .provider import Provider
-from .excs import MediaNotFound, ProviderAlreadyExists, ModelNotFound, ResourceNotFound
-from .flags import Flags as PF
-from .models import (
-    ModelFlags as MF, BaseModel,
+from feeluown.library.ai_standby import AIStandbyMatcher
+from feeluown.library.base import SearchType, ModelType
+from feeluown.library.provider import Provider
+from feeluown.library.excs import (
+    MediaNotFound, ProviderAlreadyExists, ModelNotFound, ResourceNotFound,
+)
+from feeluown.library.flags import Flags as PF
+from feeluown.library.models import (
+    ModelFlags as MF, BaseModel, SimpleSearchResult,
     BriefVideoModel, BriefSongModel, SongModel,
     LyricModel, VideoModel, BriefAlbumModel, BriefArtistModel
 )
-from .model_state import ModelState
-from .provider_protocol import (
+from feeluown.library.model_state import ModelState
+from feeluown.library.provider_protocol import (
     check_flag as check_flag_impl,
     SupportsSongLyric, SupportsSongMV, SupportsSongMultiQuality,
     SupportsVideoMultiQuality, SupportsSongWebUrl, SupportsVideoWebUrl,
 )
+from feeluown.library.standby import (
+    get_standby_score,
+    STANDBY_DEFAULT_MIN_SCORE,
+    STANDBY_FULL_SCORE,
+)
 
 if TYPE_CHECKING:
-    from .ytdl import Ytdl
-
+    from feeluown.ai import AI
+    from feeluown.library.ytdl import Ytdl
 
 logger = logging.getLogger(__name__)
 
-FULL_SCORE = 10
-MIN_SCORE = 5
 T_p = TypeVar('T_p')
 
 
@@ -38,52 +44,10 @@ def raise_(e):
     raise e
 
 
-def default_score_fn(origin, standby):
-
-    # TODO: move this function to utils module
-    def duration_ms_to_duration(ms):
-        if not ms:  # ms is empty
-            return 0
-        parts = ms.split(':')
-        assert len(parts) in (2, 3), f'invalid duration format: {ms}'
-        if len(parts) == 3:
-            h, m, s = parts
-        else:
-            m, s = parts
-            h = 0
-        return int(h) * 3600 + int(m) * 60 + int(s)
-
-    score = FULL_SCORE
-    if origin.artists_name != standby.artists_name:
-        score -= 3
-    if origin.title != standby.title:
-        score -= 2
-    if origin.album_name != standby.album_name:
-        score -= 2
-
-    if isinstance(origin, SongModel):
-        origin_duration = origin.duration
-    else:
-        origin_duration = duration_ms_to_duration(origin.duration_ms)
-    if isinstance(standby, SongModel):
-        standby_duration = standby.duration
-    else:
-        standby_duration = duration_ms_to_duration(standby.duration_ms)
-    if abs(origin_duration - standby_duration) / max(origin_duration, 1) > 0.1:
-        score -= 3
-
-    # Debug code for score function
-    # print(f"{score}\t('{standby.title}', "
-    #       f"'{standby.artists_name}', "
-    #       f"'{standby.album_name}', "
-    #       f"'{standby.duration_ms}')")
-    return score
-
-
 class Library:
     """Resource entrypoints."""
 
-    def __init__(self, providers_standby=None):
+    def __init__(self, providers_standby=None, enable_ai_standby_matcher=True):
         """
 
         :type app: feeluown.app.App
@@ -91,15 +55,20 @@ class Library:
         self._providers_standby = providers_standby
         self._providers = set()
         self.ytdl: Optional['Ytdl'] = None
+        self.ai: Optional['AI'] = None
 
         self.provider_added = Signal()  # emit(AbstractProvider)
         self.provider_removed = Signal()  # emit(AbstractProvider)
+        self.enable_ai_standby_matcher = enable_ai_standby_matcher
 
     def setup_ytdl(self, *args, **kwargs):
         from .ytdl import Ytdl
 
         self.ytdl = Ytdl(*args, **kwargs)
 
+    def setup_ai(self, ai):
+        self.ai = ai
+
     def register(self, provider):
         """register provider
 
@@ -165,91 +134,145 @@ class Library:
                         yield result
 
     async def a_search(self, keyword, source_in=None, timeout=None,
-                       type_in=None,
+                       type_in=None, return_err=False,
                        **_):
         """async version of search
 
+        .. versionchanged:: 4.1.9
+            Add `return_err` parameter.
+
         TODO: add Happy Eyeballs requesting strategy if needed
         """
         type_in = SearchType.batch_parse(type_in) if type_in else [SearchType.so]
 
+        # Wrap the search function to associate the result with source.
+        def wrap_search(pvd, kw, t):
+            def search():
+                try:
+                    res = pvd.search(kw, type_=t)
+                except Exception as e:  # noqa
+                    if return_err:
+                        logger.exception('One provider search failed')
+                        return SimpleSearchResult(
+                            q=keyword,
+                            source=pvd.identifier,  # noqa
+                            err_msg=f'{type(e)}',
+                        )
+                    raise e
+                # When a provider does not implement search method, it returns None.
+                if res is not None and (
+                    res.songs or res.albums or
+                    res.artists or res.videos or res.playlists
+                ):
+                    return res
+                return SimpleSearchResult(
+                    q=keyword, source=pvd.identifier, err_msg='结果为空')
+            return search
+
         fs = []  # future list
         for provider in self._filter(identifier_in=source_in):
             for type_ in type_in:
-                future = run_fn(partial(provider.search, keyword, type_=type_))
+                future = run_fn(wrap_search(provider, keyword, type_))
                 fs.append(future)
-
-        for future in as_completed(fs, timeout=timeout):
+        for task_ in as_completed(fs, timeout=timeout):
             try:
-                result = await future
-            except:  # noqa
-                logger.exception('search task failed')
-                continue
+                result = await task_
+            except Exception as e:  # noqa
+                logger.exception('One search task failed due to asyncio')
             else:
-                # When a provider does not implement search method, it returns None.
-                if result is not None:
-                    yield result
+                yield result
 
-    async def a_list_song_standby_v2(self, song,
-                                     audio_select_policy='>>>', source_in=None,
-                                     score_fn=None, min_score=MIN_SCORE, limit=1):
+    async def a_song_prepare_media_no_exc(self, standby, policy):
+        media = None
+        try:
+            media = await run_fn(self.song_prepare_media, standby, policy)
+        except MediaNotFound as e:
+            logger.debug(f'standby media not found: {e}')
+        except:  # noqa
+            logger.exception(f'get standby:{standby} media failed')
+        return media
+
+    async def a_list_song_standby_v2(
+            self, song, audio_select_policy='>>>', source_in=None,
+            score_fn=None, min_score=STANDBY_DEFAULT_MIN_SCORE, limit=1):
         """list song standbys and their media
 
         .. versionadded:: 3.7.8
-
         """
-
-        async def prepare_media(standby, policy):
-            media = None
-            try:
-                media = await run_fn(self.song_prepare_media, standby, policy)
-            except MediaNotFound as e:
-                logger.debug(f'standby media not found: {e}')
-            except:  # noqa
-                logger.exception(f'get standby:{standby} media failed')
-            return media
-
         if source_in is None:
             pvd_ids = self._providers_standby or [pvd.identifier for pvd in self.list()]
         else:
             pvd_ids = [pvd.identifier for pvd in self._filter(identifier_in=source_in)]
         if score_fn is None:
-            score_fn = default_score_fn
+            score_fn = get_standby_score
         limit = max(limit, 1)
 
         q = '{} {}'.format(song.title_display, song.artists_name_display)
         standby_score_list = []  # [(standby, score), (standby, score)]
-        song_media_list = []     # [(standby, media), (standby, media)]
+        song_media_list = []  # [(standby, media), (standby, media)]
+        top2_standby = []
         async for result in self.a_search(q, source_in=pvd_ids):
             if result is None:
                 continue
             # Only check the first 3 songs
-            for standby in result.songs:
+            for i, standby in enumerate(result.songs):
+                # HACK(cosven): I think the local provider should not be included,
+                #   because the search algorithm of local provider is so bad.
+                if i < 2 and standby.source != 'local':
+                    top2_standby.append(standby)
                 score = score_fn(song, standby)
-                if score == FULL_SCORE:
-                    media = await prepare_media(standby, audio_select_policy)
+                if score == STANDBY_FULL_SCORE:
+                    media = await self.a_song_prepare_media_no_exc(
+                        standby,
+                        audio_select_policy
+                    )
                     if media is None:
                         continue
-                    logger.debug(f'find full mark standby for song:{q}')
+                    logger.info(f'Find full score standby for song:{q}')
                     song_media_list.append((standby, media))
                     if len(song_media_list) >= limit:
                         # Return as early as possible to get better performance
                         return song_media_list
                 elif score >= min_score:
                     standby_score_list.append((standby, score))
-        standby_pvd_id_set = {standby.source for standby, _ in standby_score_list}
-        logger.debug(f"find {len(standby_score_list)} similar songs "
-                     f"from {','.join(standby_pvd_id_set)}")
-        # Limit try times since prapare_media is an expensive IO operation
-        max_try = len(pvd_ids) * 2
-        for standby, score in sorted(standby_score_list,
-                                     key=lambda song_score: song_score[1],
-                                     reverse=True)[:max_try]:
-            media = await prepare_media(standby, audio_select_policy)
-            if media is not None:
-                song_media_list.append((standby, media))
-                if len(song_media_list) >= limit:
-                    return song_media_list
+        if standby_score_list:
+            standby_pvd_id_set = {standby.source for standby, _ in standby_score_list}
+            logger.info(f"Find {len(standby_score_list)} similar songs "
+                        f"from {','.join(standby_pvd_id_set)}. Try to get a valid media")
+            max_per_source = 2
+            standby_score_list_2 = []
+            counter = Counter()
+            for s, score in standby_score_list:
+                if counter[s.source] >= max_per_source:
+                    continue
+                counter[s.source] += 1
+                standby_score_list_2.append((s, score))
+
+            assert len(standby_score_list_2) <= max_per_source * len(standby_pvd_id_set)
+            sorted_standby_score_list = sorted(
+                standby_score_list_2,
+                key=lambda song_score: song_score[1],
+                reverse=True,
+            )
+            for standby, _ in sorted_standby_score_list:
+                # TODO: send multiple requests at a time.
+                media = await self.a_song_prepare_media_no_exc(
+                    standby,
+                    audio_select_policy
+                )
+                if media is not None:
+                    song_media_list.append((standby, media))
+                    if len(song_media_list) >= limit:
+                        return song_media_list
+            return song_media_list
+        if self.enable_ai_standby_matcher and self.ai and top2_standby:
+            logger.info(f'Try to use AI to match standby for song {song}')
+            matcher = AIStandbyMatcher(
+                self.ai, self.a_song_prepare_media_no_exc, 60, audio_select_policy)
+            song_media_list = await matcher.match(song, top2_standby)
+            word = 'found a' if song_media_list else 'found no'
+            logger.info(f'AI {word} standby for song:{song}')
+            return song_media_list
         return song_media_list
 
     def check_flags(self, source: str, model_type: ModelType, flags: PF) -> bool:
